#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <SDL.h>

#include "skeleton.h"
#include "random.h"
#include "file.h"

#define CAPTION "skeleton of number 12x12"
#define SAVEFILE "skeleton.sav"

enum {
	BLACK_IMAGE = 10,
	CURSOR_IMAGE,
	MARK_OFF_IMAGE,
	MARK_ON_IMAGE,
	RETRY_IMAGE,
	NEXT_IMAGE,
	CLOSE_IMAGE,
	IMAGE_NUMS,
};

enum {
	EVENT_DONE = SDL_USEREVENT,
};

enum {
	PUSHED_NUMBER = 0, PUSHED_LINE = 10,
	PUSHED_USED = MAXLINES + 10,
	PUSHED_RETRY = MAXLINES + MAXLINES + 10, PUSHED_NEXT, PUSHED_CLOSE,
	PUSHED_GRID,
};

enum game_state {
	INITIAL,
	LOAD,
	SAVE,
	GENERATE,
	START_GAME,
	GAMING,
	CHECK_SOLUTION,
	COMPLETED,
	QUIT,
};

typedef struct {
	/* screen */
	SDL_Surface *screen;
	SDL_Surface *image;
	SDL_Event event;
	SDL_Rect cell;
	SDL_Rect grid;
	SDL_Rect icon;
	SDL_Rect retry;
	SDL_Rect next;
	SDL_Rect list;
	SDL_Rect close;
	Uint32 background_color;
	Uint32 line_color;
	/* board */
	board_type board;
	struct line lines[MAXLINES];
	struct number nums[MAXLINES];
	short idx[MAXLENGTH - 1];
	index_type lidx;
	short used[MAXLINES];
	unsigned short solution[MAXLINES];
	int nlines;
	/* state */
	enum game_state mode;
	int state, pushed;
	SDL_mutex *mutex;
	unsigned short ix, iy, iz;
} GlobalObject;


void reset_used(GlobalObject *go)
{
	int i;
	for (i = 0; i < go->nlines; i ++) {
		go->used[i] = 0;
	}
}

void reset_game(GlobalObject *go)
{
	reset_used(go);
	go->pushed = -1;
	go->state = 0;
}

int hittest(Uint16 x, Uint16 y, SDL_Rect *rc)
{
	return x > rc->x && x < rc->x + rc->w &&
		y > rc->y && y < rc->y + rc->h;
}


int get_hit(GlobalObject *go, Uint16 ex, Uint16 ey)
{
	int i, j, x, y;
	if (hittest(ex, ey, &go->grid)) {
		x = (ex - go->grid.x) / go->cell.w;
		y = (ey - go->grid.y) / go->cell.h;
		if (x >= 0 && x < COLS && y >= 0 && y < ROWS) {
			return PUSHED_GRID + x + COLS * y;
		}
	}
	else if (hittest(ex, ey, &go->list)) {
		x = ex - go->list.x; y = ey - go->list.y;
		i = x / go->cell.w / (MAXLENGTH + 2) * 19 + y / go->cell.w;
		if (0 <= i && i < go->nlines) {
			j = x % (go->cell.w * (MAXLENGTH + 2));
			if (j < go->cell.w) {
				return PUSHED_USED + i;
			}
			else {
				return PUSHED_LINE + i;
			}
		}
	}
	else if (hittest(ex, ey, &go->icon)) {
		i = (ex - go->icon.x) / go->cell.w;
		if (i >= 0 && i < 10) {
			return PUSHED_NUMBER + i;
		}
	}
	else if (hittest(ex, ey, &go->retry)) {
		return PUSHED_RETRY;
	}
	else if (hittest(ex, ey, &go->next)) {
		return PUSHED_NEXT;
	}
	else if (hittest(ex, ey, &go->close)) {
		return PUSHED_CLOSE;
	}
	return -1;
}


void draw_line(SDL_Surface *surface, int x1, int y1, int x2, int y2, Uint32 color)
{
	int dx, dy, s, step;
	SDL_Rect r;
	
	r.w = r.h = 1;
	dx = abs(x2 - x1);  dy = abs(y2 - y1);
	if (dx > dy) {
		if (x1 > x2) {
			step = (y1 > y2) ? 1 : -1;
			s = x1;  x1 = x2;  x2 = s;  y1 = y2;
		} else step = (y1 < y2) ? 1: -1;
		r.x = x1; r.y = y1;
		SDL_FillRect(surface, &r, color);
		s = dx >> 1;
		while (++x1 <= x2) {
			if ((s -= dy) < 0) {
				s += dx;  y1 += step;
			};
			r.x = x1; r.y = y1;
			SDL_FillRect(surface, &r, color);
		}
	} else {
		if (y1 > y2) {
			step = (x1 > x2) ? 1 : -1;
			s = y1;  y1 = y2;  y2 = s;  x1 = x2;
		} else step = (x1 < x2) ? 1 : -1;
		r.x = x1; r.y = y1;
		SDL_FillRect(surface, &r, color);
		s = dy >> 1;
		while (++y1 <= y2) {
			if ((s -= dx) < 0) {
				s += dy;  x1 += step;
			}
			r.x = x1; r.y = y1;
			SDL_FillRect(surface, &r, color);
		}
	}
}


void draw_rect(SDL_Surface *surface, SDL_Rect *rect, Uint16 w, Uint32 color)
{
	SDL_Rect rc;
	rc.x = rect->x; rc.y = rect->y;
	rc.w = rect->w; rc.h = w;
	SDL_FillRect(surface, &rc, color);
	rc.w = w; rc.h = rect->h + w;
	SDL_FillRect(surface, &rc, color);
	rc.x = rect->x + rect->w; rc.y = rect->y;
	SDL_FillRect(surface, &rc, color);
	rc.x = rect->x; rc.y = rect->y + rect->h;
	rc.w = rect->w + w; rc.h = w;
	SDL_FillRect(surface, &rc, color);
}


void draw_image(SDL_Surface *src, int index, SDL_Surface *dst, SDL_Rect *rect)
{
	SDL_Rect rc;
	rc.x = rect->w * index; rc.y = 0;
	rc.w = rect->w; rc.h = rect->h;
	SDL_BlitSurface(src, &rc, dst, rect);
}


void display_icon(GlobalObject *go)
{
	SDL_Surface *surface = go->screen;
	SDL_Surface *image = go->image;
	SDL_Rect *cell = &go->cell;
	SDL_Rect *icon = &go->icon;
	SDL_Rect rc;
	int i;
	
	rc.x = icon->x; rc.y = icon->y;
	rc.w = cell->w; rc.h = cell->h;
	if (go->state == 0) {
		draw_image(image, CURSOR_IMAGE, surface, &rc);
	}
	for (i = 0; i < 9; i ++) {
		rc.x += cell->w;
		if (go->state == i + 1) {
			draw_image(image, CURSOR_IMAGE, surface, &rc);
		}
		draw_image(image, i + 1, surface, &rc);
	}
	
	rc = *icon;
	rc.x -= 2; rc.y -= 2;
	rc.w += 2; rc.h += 2;
	draw_rect(surface, &rc, 2, go->line_color);
	
	rc.x = icon->x + cell->w; rc.y = icon->y;
	rc.w = 1; rc.h = cell->h;
	for (i = 0; i < 9; i ++) {
		SDL_FillRect(surface, &rc, go->line_color);
		rc.x += cell->w;
	}
	
	rc = go->retry;
	if (go->pushed == PUSHED_RETRY) { rc.x += rc.w / 10 + 1; rc.y += rc.h / 10 + 1; }
	draw_image(image, RETRY_IMAGE, surface, &rc);
	
	rc = go->next;
	if (go->pushed == PUSHED_NEXT) { rc.x += rc.w / 10 + 1; rc.y += rc.h / 10 + 1; }
	draw_image(image, NEXT_IMAGE, surface, &rc);
	
	rc = go->close;
	if (go->pushed == PUSHED_CLOSE) { rc.x += rc.w / 10 + 1; rc.y += rc.h / 10 + 1; }
	draw_image(image, CLOSE_IMAGE, surface, &rc);
}


void display_numbers(GlobalObject *go)
{
	SDL_Surface *surface = go->screen;
	SDL_Surface *image = go->image;
	SDL_Rect *cell = &go->cell;
	SDL_Rect *list = &go->list;
	struct number *nums = go->nums;
	short *used = go->used;
	int nlines = go->nlines;
	SDL_Rect rc;
	int i, j, k, x, y, w, s;
	
	s = go->state - 10;
	
	x = list->x; y = list->y; w = cell->w;
	rc.w = cell->w; rc.h = cell->h;
	for (i = 0, j = 0; i < nlines; i ++, j ++) {
		if (j == 19) {
			x += w * (MAXLENGTH + 2);
			y = list->y;
			j = 0;
		}
		rc.x = x; rc.y = y;
		draw_image(image, used[i] ? MARK_ON_IMAGE : MARK_OFF_IMAGE, surface, &rc);
		rc.x += w;
		for (k = 0; k < nums[i].length; k ++) {
			if (i == s) {
				draw_image(image, CURSOR_IMAGE, surface, &rc);
			}
			draw_image(image, nums[i].digit[k], surface, &rc);
			rc.x += w;
		}
		y += cell->h;
	}
}


void display_completed(GlobalObject *go, int count)
{
	SDL_Surface *surface = go->screen;
	SDL_Rect *cell = &go->cell;
	SDL_Rect *grid = &go->grid;
	SDL_Rect *list = &go->list;
	struct line *lines = go->lines;
	unsigned short *solution = go->solution;
	int i, j, x1, y1, x2, y2, w, h;
	
	if (count < go->nlines) {
		x1 = list->x + (count / 19) * cell->w * (MAXLENGTH + 2) + cell->w / 2;
		y1 = list->y + (count % 19) * cell->h + cell->h / 2;
		x2 = grid->x + lines[solution[count] - 1].x * cell->w + cell->w / 2;
		y2 = grid->y + lines[solution[count] - 1].y * cell->h + cell->h / 2;
		draw_line(surface, x1, y1, x2, y2, rand());
	}
	else {
		w = cell->w / 2; h = cell->h / 2;
		x1 = list->x; y1 = list->y;
		for (i = 0, j = 0; i < count; i ++, j ++) {
			if (j == 19) {
				x1 += cell->w * (MAXLENGTH + 2);
				y1 = list->y;
				j = 0;
			}
			x2 = grid->x + lines[solution[i] - 1].x * cell->w + w;
			y2 = grid->y + lines[solution[i] - 1].y * cell->h + h;
			draw_line(surface, x1 + w, y1 + h, x2, y2, rand());
			y1 += cell->h;
		}
	}
}


void display_cursor(GlobalObject *go)
{
	SDL_Rect *cell = &go->cell;
	SDL_Rect *grid = &go->grid;
	int i, j, x, y;
	SDL_Rect rc;
	
	SDL_GetMouseState(&x, &y);
	if (! hittest(x, y, grid)) return;
	x = (x - grid->x) / cell->w;
	y = (y - grid->y) / cell->h;
	if (go->board[y][x] == BLACK) return;
	if (go->state < 10) {
		rc.x = grid->x + x * cell->w;
		rc.y = grid->y + y * cell->h;
		rc.w = cell->w; rc.h = cell->h;
		draw_image(go->image, CURSOR_IMAGE, go->screen, &rc);
	}
	else {
		j = go->state - 10;
		if ((i = get_vertical_line(go->lidx, x, y) - 1) >= 0 &&
			can_set_number(go->board, &go->lines[i], &go->nums[j])) {
			rc.x = grid->x + go->lines[i].x * cell->w;
			rc.y = grid->y + go->lines[i].y * cell->h;
			rc.w = cell->w; rc.h = cell->h;
			for (j = 0; j < go->lines[i].length; j ++) {
				draw_image(go->image, CURSOR_IMAGE, go->screen, &rc);
				rc.y += cell->h;
			}
		}
		else if ((i = get_horizontal_line(go->lidx, x, y) - 1) >= 0 &&
			can_set_number(go->board, &go->lines[i], &go->nums[j])) {
			rc.x = grid->x + go->lines[i].x * cell->w;
			rc.y = grid->y + go->lines[i].y * cell->h;
			rc.w = cell->w; rc.h = cell->h;
			for (j = 0; j < go->lines[i].length; j ++) {
				draw_image(go->image, CURSOR_IMAGE, go->screen, &rc);
				rc.x += cell->w;
			}
		}
	}
}


void display_grid(GlobalObject *go)
{
	SDL_Surface *surface = go->screen;
	SDL_Rect *cell = &go->cell;
	SDL_Rect *grid = &go->grid;
	int x, y;
	SDL_Rect rc;
	
	// draw grid
	
	rc = *grid;
	rc.x -= 2; rc.y -= 2;
	rc.w += 2; rc.h += 2;
	draw_rect(surface, &rc, 2, go->line_color);
	
	rc.y = grid->y; rc.w = 1; rc.h = grid->h;
	for (x = cell->w; x < grid->w; x += cell->w) {
		rc.x = grid->x + x;
		SDL_FillRect(surface, &rc, go->line_color);
	}
	
	rc.x = grid->x; rc.w = grid->w; rc.h = 1;
	for (y = cell->h; y < grid->h; y += cell->h) {
		rc.y = grid->y + y;
		SDL_FillRect(surface, &rc, go->line_color);
	}
}


void display_board(GlobalObject *go)
{
	SDL_Rect *cell = &go->cell;
	SDL_Rect *grid = &go->grid;
	int x, y;
	SDL_Rect rc, src;
	
	display_cursor(go);
	
	// draw cells
	rc.w = cell->w; rc.h = cell->h; rc.y = grid->y;
	src.w = cell->w, src.h = cell->h;
	for (y = 0; y < ROWS; y ++) {
		rc.x = grid->x;
		for (x = 0; x < COLS; x ++) {
			if (1 <= go->board[y][x] && go->board[y][x] <= 9) {
				src.x = cell->w * go->board[y][x]; src.y = 0;
				draw_image(go->image, go->board[y][x], go->screen, &rc);
			}
			else if (go->board[y][x] == BLACK) {
				draw_image(go->image, BLACK_IMAGE, go->screen, &rc);
			}
			rc.x += cell->w;
		}
		rc.y += cell->h;
	}
}


void on_active(GlobalObject *go, SDL_ActiveEvent *event)
{
	if (event->gain == 0) {
		go->pushed = -1;
	}
}


void on_button_down(GlobalObject *go, SDL_MouseButtonEvent *event)
{
	go->pushed = get_hit(go, event->x, event->y);
}


void on_mouse_motion(GlobalObject *go, SDL_MouseMotionEvent *event)
{
	int h;
	h = get_hit(go, event->x, event->y);
	if (h < 0 || h != go->pushed) {
		go->pushed = -1;
	}
}


void on_button_up(int mode, GlobalObject *go, SDL_MouseButtonEvent *event)
{
	int i, j, x, y, h;
	
	h = get_hit(go, event->x, event->y);
	if (h < 0 || go->pushed < 0 || h != go->pushed) {
		go->pushed = -1;
		return;
	}
	if (h >= PUSHED_GRID) {
		if (mode == GAMING) {
			h -= PUSHED_GRID;
			x = h % COLS; y = h / COLS;
			if (0 <= go->state && go->state <= 9) {
				if (go->board[y][x] != BLACK) {
					go->board[y][x] = go->state;
				}
			}
			else {
				i = go->state - 10;
				if (0 <= i && i < go->nlines) {
					if ((j = get_vertical_line(go->lidx, x, y) - 1) >= 0 &&
						set_number(go->board, &go->lines[j], &go->nums[i])) {
						go->used[i] = 1;
					}
					else if ((j = get_horizontal_line(go->lidx, x, y) - 1) >= 0 &&
						set_number(go->board, &go->lines[j], &go->nums[i])) {
						go->used[i] = 1;
					}
				}
			}
		}
	}
	else if (h == PUSHED_CLOSE) {
		if (mode == GAMING) {
			save_skeleton_state(SAVEFILE, go->ix, go->iy, go->iz, go->board, go->nums, go->used, go->nlines);
		}
		go->mode = QUIT;
	}
	else if (h == PUSHED_NEXT) {
		if (mode == GAMING) {
			go->mode = CHECK_SOLUTION;
		}
		else {
			go->mode = INITIAL;
		}
	}
	else if (h == PUSHED_RETRY) {
		if (mode == GAMING) {
			reset_board(go->board);
			reset_game(go);
		}
		else {
			go->mode = INITIAL;
		}
	}
	else if (h >= PUSHED_USED) {
		if (mode == GAMING) {
			i = h - PUSHED_USED;
			go->used[i] = 1 - go->used[i];
		}
	}
	else if (h >= PUSHED_LINE) {
		if (mode == GAMING) {
			i = h - PUSHED_LINE;
			go->state = i + 10;
		}
	}
	else if (h >= PUSHED_NUMBER) {
		if (mode == GAMING) {
			go->state = h;
		}
	}
	go->pushed = -1;
}


void update_caption(GlobalObject *go)
{
	char caption[] = CAPTION " (30000 30000 30000)";
	
	sprintf(caption, CAPTION " (%u %u %u)", go->ix, go->iy, go->iz);
#ifndef NDEBUG
	printf("seed %d %d %d\n", go->ix, go->iy, go->iz);
#endif
	SDL_WM_SetCaption(caption, NULL);
}


int set_terminate(GlobalObject *go)
{
	while (SDL_mutexP(go->mutex) == -1) {
		SDL_Delay(10);
	}
	go->state = 1;
	SDL_mutexV(go->mutex);
	return 0;
}


int check_terminate(void *arg)
{
	GlobalObject *go = arg;
	int result;
	
	if (SDL_mutexP(go->mutex) == -1) return 1;
	result = go->state;
	SDL_mutexV(go->mutex);
	return result;
}


int generate_thread(void *arg)
{
	int i, r;
	GlobalObject *go = arg;
	SDL_Event ev = {0};
	
	while (! check_terminate(go)) {
		rand_get_seed(&go->ix, &go->iy, &go->iz);
		init_board(go->board);
		while (select_dig(go->board));
		if (check_terminate(go)) return 1;
		if (check_link_count(go->board)) continue;
		if (check_black_box(go->board)) continue;
		if (check_split(go->board)) continue;
		if (check_terminate(go)) return 1;
		go->nlines = set_lines(go->board, go->lines, MAXLINES);
		set_number_index(go->lines, go->idx, go->nlines);
		set_line_index(go->lines, go->nlines, go->lidx);
		if (check_terminate(go)) return 1;
		for (i = 0; i < 30; i ++) {
			r = fill_number(go->board, go->lines, go->nums, go->nlines, 10, check_terminate, go);
			if (r < 0) return 1;
			if (r) break;
			if (check_duplicate_num(go->nums, go->nlines)) continue;
			r = check_duplicate_solution(go->board, go->lines, go->nums, go->idx, go->solution, go->nlines, check_terminate, go);
			if (r < 0) return 1;
			if (r) continue;
			reset_board(go->board);
			r = check_solve(go->nlines, go->board, go->lines, go->nums, go->idx, go->nlines, check_terminate, go);
			if (r < 0) return 1;
			if (r == 1) {
				ev.type = EVENT_DONE;
				SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, 0);
				return 0;
			}
		}
	}
	return 1;
}


int main_loop(GlobalObject *go)
{
	enum game_state mode;
	SDL_Event event;
	int quit, count;
	SDL_Thread *th = NULL;
	
	quit = count = 0;
	rand_set_seed(go->ix, go->iy, go->iz);
	go->mode = (go->state) ? INITIAL : LOAD;
	while (! quit) {
		mode = go->mode;
		switch (mode) {
		case LOAD:
			go->nlines = load_skeleton_state(SAVEFILE, &go->ix, &go->iy, &go->iz, go->board, go->nums, go->used);
			rand_normalize_seed(&go->ix, &go->iy, &go->iz);
			if (go->nlines > 0 && go->nlines == set_lines(go->board, go->lines, MAXLINES)) {
				set_number_index(go->lines, go->idx, go->nlines);
				set_line_index(go->lines, go->nlines, go->lidx);
				go->mode = START_GAME;
				break;
			}
			go->mode = INITIAL;
			break;
		case INITIAL:
			if (th) {
				set_terminate(go);
				SDL_WaitThread(th, NULL);
				th = NULL;
			}
			reset_game(go);
			SDL_WM_SetCaption(CAPTION, NULL);
			go->mode = GENERATE;
			th = SDL_CreateThread(generate_thread, go);
			if (! th) return 0;
			break;
		case GENERATE:
			SDL_FillRect(go->screen, 0, go->background_color);
			display_grid(go);
			display_icon(go);
			SDL_Flip(go->screen);
			if (SDL_WaitEvent(&event)) {
				switch (event.type) {
				case SDL_ACTIVEEVENT:
					on_active(go, (SDL_ActiveEvent *) &event);
					break;
				case SDL_QUIT:
					go->mode = QUIT;
					break;
				case EVENT_DONE:
					SDL_WaitThread(th, NULL);
					th = NULL;
					go->mode = START_GAME;
					break;
				case SDL_MOUSEBUTTONDOWN:
					on_button_down(go, (SDL_MouseButtonEvent *) &event);
					break;
				case SDL_MOUSEBUTTONUP:
					on_button_up(mode, go, (SDL_MouseButtonEvent *) &event);
					break;
				case SDL_MOUSEMOTION:
					on_mouse_motion(go, (SDL_MouseMotionEvent *) &event);
					break;
				default:
					break;
				}
			}
			break;
		case START_GAME:
			update_caption(go);
			reset_game(go);
			go->mode = GAMING;
			break;
		case GAMING:
			SDL_FillRect(go->screen, 0, go->background_color);
			display_board(go);
			display_grid(go);
			display_numbers(go);
			display_icon(go);
			SDL_Flip(go->screen);
			if (SDL_WaitEvent(&event)) {
				switch (event.type) {
				case SDL_ACTIVEEVENT:
					on_active(go, (SDL_ActiveEvent *) &event);
					break;
				case SDL_QUIT:
					save_skeleton_state(SAVEFILE, go->ix, go->iy, go->iz, go->board, go->nums, go->used, go->nlines);
					go->mode = QUIT;
					break;
				case SDL_MOUSEBUTTONDOWN:
					on_button_down(go, (SDL_MouseButtonEvent *) &event);
					break;
				case SDL_MOUSEBUTTONUP:
					on_button_up(mode, go, (SDL_MouseButtonEvent *) &event);
					break;
				case SDL_MOUSEMOTION:
					on_mouse_motion(go, (SDL_MouseMotionEvent *) &event);
				default:
					break;
				}
			}
			break;
		case CHECK_SOLUTION:
			if (check_solution(go->board, go->lines, go->nums, go->idx, go->solution, go->nlines)) {
				remove(SAVEFILE);
				count = 0;
				go->mode = COMPLETED;
			}
			else {
				go->mode = GAMING;
			}
			break;
		case COMPLETED:
			SDL_FillRect(go->screen, 0, go->background_color);
			display_board(go);
			display_grid(go);
			display_numbers(go);
			display_icon(go);
			display_completed(go, count);
			SDL_Flip(go->screen);
			if (count < go->nlines) {
				count ++;
			}
			SDL_Delay(100);
			if (SDL_PollEvent(&event)) {
				switch (event.type) {
				case SDL_ACTIVEEVENT:
					on_active(go, (SDL_ActiveEvent *) &event);
					break;
				case SDL_QUIT:
					go->mode = QUIT;
					break;
				case SDL_MOUSEBUTTONDOWN:
					on_button_down(go, (SDL_MouseButtonEvent *) &event);
					break;
				case SDL_MOUSEBUTTONUP:
					on_button_up(mode, go, (SDL_MouseButtonEvent *) &event);
					break;
				case SDL_MOUSEMOTION:
					on_mouse_motion(go, (SDL_MouseMotionEvent *) &event);
				default:
					break;
				}
			}
			break;
		case QUIT:
			if (th) {
				set_terminate(go);
				SDL_WaitThread(th, NULL);
				th = NULL;
			}
			quit = 1;
			break;
		default:
			return 0;
		}
	}
	return 0;
}


Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
	SDL_PixelFormat *fmt = surface->format;
	Uint8 bpp = fmt->BytesPerPixel;
	Uint8 *p = (Uint8 *) surface->pixels + y * surface->pitch + x * bpp;
	
	if (bpp == 3) {
		if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
			return p[0] << 16 | p[1] << 8 | p[2];
		else
			return p[0] | p[1] << 8 | p[2] << 16;
	}
	else {
		return *p;
	}
}


int set_color(GlobalObject *go)
{
	Uint32 bg_color, line_color;
	Uint8 r, g, b;
	
	if (SDL_LockSurface(go->image)) return 1;
	bg_color = getpixel(go->image, 0, 0);
	line_color = getpixel(go->image, go->cell.w * BLACK_IMAGE, 0);
	SDL_UnlockSurface(go->image);
	
	SDL_GetRGB(bg_color, go->image->format, &r, &g, &b);
	SDL_SetColorKey(go->image, SDL_SRCCOLORKEY, SDL_MapRGB(go->image->format, r, g, b));
	go->background_color = SDL_MapRGB(go->screen->format, r, g, b);
	
	SDL_GetRGB(line_color, go->image->format, &r, &g, &b);
	go->line_color = SDL_MapRGB(go->screen->format, r, g, b);
	
	return 0;
}


int init(const char *filename, GlobalObject *go)
{
	SDL_Surface *screen, *image;
	
	image = SDL_LoadBMP(filename);
	if (! image) {
		fprintf(stderr, "Could not load bmp.\n");
		return 1;
	}
	
	go->cell.x = go->cell.y = 0;
	go->cell.w = image->w / IMAGE_NUMS;
	go->cell.h = image->h;
	
	go->grid.x = go->cell.w;
	go->grid.y = go->cell.h * 2;
	go->grid.w = go->cell.w * COLS;
	go->grid.h = go->cell.h * ROWS;
	
	go->icon.x = go->cell.w * 2;
	go->icon.y = go->cell.h * 16;
	go->icon.w = go->cell.w * 10;
	go->icon.h = go->cell.h;
	
	go->retry.x = go->cell.w * 2;
	go->retry.y = go->cell.h * 18;
	go->retry.w = go->cell.w;
	go->retry.h = go->cell.h;
	
	go->next.x = go->cell.w * 11;
	go->next.y = go->cell.h * 18;
	go->next.w = go->cell.w;
	go->next.h = go->cell.h;
	
	go->list.x = go->cell.w * (COLS + 2);
	go->list.y = go->cell.h;
	go->list.w = go->cell.w * (MAXLENGTH + 2) * 3;
	go->list.h = go->cell.h * 19;
	
	go->close.x = go->cell.w * 34;
	go->close.y = go->cell.h * 20;
	go->close.w = go->cell.w;
	go->close.h = go->cell.h;
	
	screen = SDL_SetVideoMode(35 * go->cell.w, 21 * go->cell.h,
		8, SDL_SWSURFACE|SDL_ANYFORMAT);
	if (! screen) {
		SDL_FreeSurface(image);
		fprintf(stderr, "Could not set video.\n");
		return 1;
	}
	if (image->format->palette && screen->format->palette) {
		SDL_SetColors(screen, image->format->palette->colors, 0,
					  image->format->palette->ncolors);
	}
	
	go->screen = screen;
	go->image = image;
	SDL_Flip(screen);
	
	if (set_color(go)) {
		fprintf(stderr, "Could not set color.\n");
		SDL_FreeSurface(image);
		return 1;
	}
	
	go->mutex = SDL_CreateMutex();
	
	return 0;
}


int finalize(GlobalObject *go)
{
	SDL_DestroyMutex(go->mutex);
	SDL_FreeSurface(go->image);
	return 0;
}


void set_seed(int argc, char *argv[], GlobalObject *go)
{
	int x, y, z;
	if (argc > 3) {
		x = atoi(argv[1]);
		y = atoi(argv[2]);
		z = atoi(argv[3]);
		go->state = 1;
	}
	else {
		y = time(NULL);
		x = SDL_GetTicks();
		z = x * y;
		go->state = 0;
	}
	go->ix = x;
	go->iy = y;
	go->iz = z;
	rand_normalize_seed(&go->ix, &go->iy, &go->iz);
}


int main(int argc, char *argv[])
{
	GlobalObject go = {0};
	
	if(SDL_Init(SDL_INIT_VIDEO) == -1) {
		fprintf(stderr, "Could not initialize SDL: %s.\n", SDL_GetError());
		return -1;
	}
	if(init("skeleton.bmp", &go)) {
		fprintf(stderr, "Could not initialize.\n");
		return -1;
	}
	set_seed(argc, argv, &go);
	main_loop(&go);
	finalize(&go);
	SDL_Quit();
	
#ifndef NDEBUG
	printf("Quit\n");
#endif
	return 0;
}
